Arc

Struct Arc 

pub struct Arc<T: Poolable> { /* private fields */ }
A poolable drop-in replacement for std::sync::Arc.

This Arc pools its allocations, reducing overhead when frequently creating and destroying Arc instances. When the last strong reference is dropped, the allocation is returned to the pool instead of being freed.

Example

use poolshark::global::arc::Arc;
use poolshark::global::RawPool;

let pool: RawPool<Arc<String>> = RawPool::new(1024, 1);
let arc1 = Arc::new(&pool, "hello".to_string());
let arc2 = arc1.clone();  // Reference counting works normally
drop(arc1);
drop(arc2);  // Returns to pool when last reference drops

Differences from std::sync::Arc

  • Requires a pool to be created from
  • Returns to pool when dropped (if strong_count == 1)
  • Slightly larger memory footprint (stores pool pointer)

Implementations

Source

impl<T: Poolable> Arc<T>

pub fn new(pool: &RawPool<Self>, v: T) -> Self

Allocate a new arc from the specified pool.

Returns an arc containing v.

pub fn get_mut(&mut self) -> Option<&mut T>

Get a mutable reference to the inner value if the Arc is unique.

Returns None if the Arc is not unique (strong_count > 1).

pub fn strong_count(&self) -> usize

Return the strong reference count of the arc.

pub fn as_ptr(&self) -> *const (WeakPool<Self>, T)

Return the arc as a raw pointer.

Source

impl<T: Poolable + Clone> Arc<T>

pub fn make_mut<'a>(&'a mut self) -> &'a mut T

Get a mutable reference to the inner value, cloning if necessary.

If the Arc is not unique, this will clone the inner value.

Source

impl<T: Poolable + Clone> Arc<T>

pub fn downgrade(&self) -> Weak<T>

Downgrade the Arc to a weak pointer.

pub fn weak_count(&self) -> usize

Return the weak reference count of the arc.