insert

Function insert 

pub fn insert<T: IsoPoolable>(t: T) -> Option<T>
Insert a T into the pool.

If there is no space in the pool available to hold T then return it, otherwise return None. T will be reset before it is inserted into the pool. Reset must ensure that T is EMPTY.